import { Component, Input, Output, EventEmitter, ViewChildren, QueryList } from "@angular/core";

import { ParamDefinition } from "jalhyd";

import { NgParameter, ParamRadioConfig } from "../../formulaire/ngparam";
import { FieldSet } from "../../formulaire/fieldset";
import { ParamFieldLineComponent } from "../param-field-line/param-field-line.component";
import { BaseComponent } from "../base/base.component";
import { SelectEntry } from "../../formulaire/select-entry";
import { Field } from "../../formulaire/field";
import { InputField } from "../../formulaire/input-field";
import { SelectField } from "../../formulaire/select-field";
import { CheckField } from "../../formulaire/check-field";

@Component({
    selector: "field-set",
    templateUrl: "./field-set.component.html",
    styles: [`.fieldset_title {
        font-weight: bold;
        margin: 0.3em 0 0.2em;
        padding: 0.3em;
        background-color: #eeeeee;
        }`,
        `.radio_param_header {
        width: 10em;
        }`
    ]
})
export class FieldSetComponent extends BaseComponent {
    /**
    * field set attribute
    */
    private _fieldSet: FieldSet;

    @Input()
    private set fieldSet(fs: FieldSet) {
        this._fieldSet = fs;
    }

    @ViewChildren(ParamFieldLineComponent)
    private _paramComponents: QueryList<ParamFieldLineComponent>;

    /**
     * événément de changement de validité
     */
    @Output()
    private onValid = new EventEmitter();

    /**
     * flag de validité de la saisie
     */
    private _isValid: boolean = false;

    private get fields() {
        return this._fieldSet.fields;
    }

    private hasRadioFix(): boolean {
        if (this._fieldSet.hasInputs)
            switch (this._fieldSet.getInput(0).radioConfig) {
                case ParamRadioConfig.FIX:
                    return false;

                default:
                    return true;
            }
        return false;
    }

    private hasRadioVar(): boolean {
        if (this._fieldSet.hasInputs)
            switch (this._fieldSet.getInput(0).radioConfig) {
                case ParamRadioConfig.VAR:
                case ParamRadioConfig.CAL:
                    return true;

                default:
                    return false;
            }
        return false;
    }

    private hasRadioCal(): boolean {
        if (this._fieldSet.hasInputs)
            switch (this._fieldSet.getInput(0).radioConfig) {
                case ParamRadioConfig.CAL:
                    return true;

                default:
                    return false;
            }
        return false;
    }

    private get title(): string {
        if (this._fieldSet == undefined)
            return "fs undefined";
        if (this._fieldSet.label == undefined)
            return "label undefined";
        return this._fieldSet.label;
    }

    /**
     * détermine si un Field est du type InputField
     */
    private isInputField(f: Field): boolean {
        return f instanceof InputField && f.isDisplayed;
    }

    /**
     * détermine si un Field est du type SelectField
     */
    private isSelectField(f: Field): boolean {
        return f instanceof SelectField && f.isDisplayed;
    }

    /**
     * détermine si un Field est du type CheckField
     */
    private isCheckField(f: Field): boolean {
        return f instanceof CheckField && f.isDisplayed;
    }

    /*
     * gestion des événements clic sur les radios :
     * réception d'un message du composant enfant (param-field)
     * cf. https://angular.io/guide/component-interaction#parent-listens-for-child-event
     */
    private onRadioClick(info: string) {
        // on renvoie l'info au parent
        this.onRadio.emit(info);
    }

    /**
     * événément de changement d'état d'un radio
     */
    @Output()
    private onRadio = new EventEmitter<string>();

    /**
     * réception d'un événement d'un select
     */
    private onSelectChanged(val: SelectEntry) {
        this.selectChange.emit(val); // on transmet au parent
    }

    public get isValid() {
        return this._isValid;
    }

    /**
     * calcul de la validité de tous les ParamFieldLineComponent de la vue
     */
    private updateValidity() {
        if (this._paramComponents != undefined)
            this._isValid = this._paramComponents.reduce(
                // callback
                (
                    // accumulator (valeur précédente du résultat)
                    acc,
                    // currentValue (élément courant dans le tableau)
                    param,
                    // currentIndex (indice courant dans le tableau)
                    currIndex,
                    // array (tableau parcouru)
                    array
                ) => {
                    return acc && param.isValid;
                }
                // valeur initiale
                , true);
    }

    protected afterFirstViewChecked() {
        this.updateValidity();
    }

    /**
     * réception d'un événement de validité de ParamFieldLineComponent
     */
    private onParamLineValid(event: boolean) {
        this.updateValidity();
        this.onValid.emit();
    }

    /**
     * événément de changement d'état d'un select
     */
    @Output()
    private selectChange = new EventEmitter<SelectEntry>();
}
