import { FormulaireElement } from "../formulaire-element";
import { DependencyCondition } from "./dependency-condition";

/** 
 * Dépendance entre un élément maître et un élément esclave
 * Si le maître vérifie une condition (masterCondition), alors la dépendance est appliquée à l'élément esclave,
 * cad que son affichage, sa valeur, ... sont modifiés
 */
export abstract class Dependency {
    private _master: FormulaireElement;

    private _masterCondition: DependencyCondition;

    constructor(m: FormulaireElement, mc: DependencyCondition) {
        this._master = m;
        this._masterCondition = mc;
    }

    public get masterElement(): FormulaireElement {
        return this._master;
    }

    public get masterCondition(): DependencyCondition {
        return this._masterCondition;
    }

    public toString(): string {
        return "master=" + this._master.toString() + "\n  " + this._masterCondition.toString();
    }

    public abstract clone(master: FormulaireElement): Dependency;
}
