import { ComputeNodeType } from "jalhyd";

import { FormulaireElement } from "./formulaire-element";
import { FieldSet } from "./fieldset";
import { Dependency } from "./dependency/dependency";
import { StringMap } from "../stringmap";
import { FormulaireDefinition } from "./formulaire-definition";
import { FormulaireNode } from "./formulaire-node";

export class FieldsetContainer extends FormulaireElement {
    private _templates: FieldSet[];

    private _localisation: StringMap;

    public title: string

    constructor(parentForm: FormulaireDefinition) {
        super(ComputeNodeType.None, parentForm);
        this._templates = [];
    }

    /**
     * cherche un FormulaireNode par son id de conf
     */
    public getFormulaireNodeById(id: string): FormulaireNode {
        const res = super.getFormulaireNodeById(id);
        if (res !== undefined)
            return res;

        for (const t of this._templates) {
            const res = t.getFormulaireNodeById(id);
            if (res !== undefined)
                return res;
        }

        return undefined;
    }

    private checkTemplate(fs: FieldSet) {
        if (!fs.isTemplate)
            throw new Error(`le Fieldset ${fs.id} n'est pas un template`);
    }

    public addTemplate(fs: FieldSet) {
        if (this.hasTemplate(fs))
            console.log(`Warning : le Fieldset template ${fs.id} a déjà été ajouté`);
        else
            this._templates.push(fs);
    }

    private hasTemplate(fs: FieldSet): boolean {
        this.checkTemplate(fs);

        for (const f of this._templates)
            if (f.id === fs.id)
                return true;
        return false;
    }

    private getTemplate(id: string) {
        for (const f of this._templates)
            if (f.id === id)
                return f;
        return undefined;
    }

    public addFromTemplate(templId: string) {
        const templ: FieldSet = this.getTemplate(templId);
        const inst = templ.instanciateTemplate();
        this.fieldsets.push(inst);
        inst.applyDependencies();
        this.updateLocalisation()
    }

    public get fieldsets(): FieldSet[] {
        return this.kids as FieldSet[];
    }

    public get templates(): FieldSet[] {
        return this._templates;
    }

    public parseConfig(json: {}, data?: {}) {
        this._confId = json["id"];

        const templs: string[] = json["templates"];
        for (const t of templs)
            // this.addTemplate(this.getFieldsetTemplate(t));
            this.addTemplate(data[t]);
    }

    public parseDependencies(json: {}) {
        super.parseDependencies(json);

        for (const t of this._templates)
            t.parseDependencies(json);
    }

    protected verifyDependency(d: Dependency): boolean {
        return true;
    }

    public updateLocalisation(loc?: StringMap) {
        if (loc == undefined)
            loc = this._localisation;
        else
            this._localisation = loc;

        super.updateLocalisation(loc);

        for (let t of this._templates)
            t.updateLocalisation(loc);
    }

    /**
     * copie des membres
     */
    protected copyMembers(n: FieldsetContainer) {
        super.copyMembers(n);
        n._localisation = this._localisation;
        n.title = this.title;
    }

    /**
     * crée une nouvelle instance
     */
    protected clone(): FieldsetContainer {
        return new FieldsetContainer(this.parentForm);
    }
}
