import { CalculatorType, ComputeNodeType } from "jalhyd";

import { FormulaireElement } from "./formulaire-element";
import { Dependency } from "./dependency/dependency";
import { DependencyConditionType } from "./dependency/dependency-condition";
import { Field } from "./field";
import { CheckField } from "./check-field";
import { SelectField } from "./select-field";
import { NgParameter } from "./ngparam";
import { ParamService } from "../services/param/param.service";
import { ApplicationSetupService } from "../services/app-setup/app-setup.service";

export class FieldSet extends FormulaireElement {
    public get fields(): Field[] {
        return this.kids as Field[];
    }

    /**
     * crée une nouvelle instance
     */
    protected clone(): FieldSet {
        return new FieldSet(this.computeNodeType, this.parentForm);
    }

    public instanciateTemplate(): FieldSet {
        const res = this.getDeepClone() as FieldSet;
        this.copyDependencies(res, res);
        return res;
    }

    public addField(f: Field) {
        this.fields.push(f);
    }

    public get hasInputs(): boolean {
        for (let f of this.fields)
            if (f instanceof NgParameter)
                return true;
        return false;
    }

    public getInput(i: number): NgParameter {
        let n = 0;
        for (let f of this.fields) {
            if (f instanceof NgParameter) {
                if (n == i)
                    return f;
                n++;
            }
        }
        return undefined;
    }

    protected verifyDependency(d: Dependency): boolean {
        throw "FieldSet.verifyDependency() : type de condition '" + DependencyConditionType[d.masterCondition.type] + "' non pris en charge";
    }

    public get isValid(): boolean {
        let res: boolean = true;
        for (let f of this.fields)
            res = res && f.isValid;
        return res;
    }

    private parse_check(json: {}, default_node_type: ComputeNodeType): CheckField {
        const nt: string = json["nodeType"];
        let node_type: ComputeNodeType = nt == undefined ? default_node_type : ComputeNodeType[nt];

        let res: CheckField = new CheckField(node_type, this.parentForm, this.isTemplate);
        res.parseConfig(json)
        return res;
    }

    private parse_select(json: {}, default_node_type: ComputeNodeType): SelectField {
        const nt: string = json["nodeType"];
        let node_type: ComputeNodeType = nt == undefined ? default_node_type : ComputeNodeType[nt];

        let res: SelectField = new SelectField(node_type, this.parentForm, this.isTemplate);
        res.parseConfig(json);
        return res;
    }

    private parse_input(json: {}, default_calc_type: CalculatorType, default_node_type: ComputeNodeType, default_radio_config: string,
        paramService: ParamService, appSetupService: ApplicationSetupService): NgParameter {
        let input_id: string = json["id"];

        const ct: string = json["calcType"];
        let calc_type: CalculatorType = ct == undefined ? default_calc_type : CalculatorType[ct];

        const nt: string = json["nodeType"];
        let node_type: ComputeNodeType = nt == undefined ? default_node_type : ComputeNodeType[nt];

        let res: NgParameter = paramService.createParameter(calc_type, node_type, input_id, this.parentForm, this.isTemplate);
        if (res == undefined)
            throw new Error(`pas de paramètre '${input_id}' trouvé dans le noeud ${ComputeNodeType[node_type]} (${node_type}) ou ComputeNodeType.None`);

        res.parseConfig(json, { "appSetupService": appSetupService, "radioConfig": default_radio_config });

        return res;
    }

    public parseConfig(json: {}, data?: {}) {
        const paramService: ParamService = data["paramService"];
        const appSetupService: ApplicationSetupService = data["appSetupService"];

        this._confId = json["id"];

        const ct: string = json["calcType"];
        let calc_type: CalculatorType = ct == undefined ? this.parentForm.calculatorType : CalculatorType[ct];

        const fields = json["fields"];
        for (const field_index in fields) {
            const field = fields[field_index];

            if (field["type"] === "input") {
                const default_radio_config = json["option"];
                const param = this.parse_input(field, calc_type, this.computeNodeType, default_radio_config, paramService, appSetupService);
                this.addField(param);
            } else if (field["type"] === "select") {
                const param = this.parse_select(field, this.computeNodeType);
                this.addField(param);
            } else if (field["type"] === "check") {
                const param = this.parse_check(field, this.computeNodeType);
                this.addField(param);
            }
        }
    }

    public parseDependencies(json: {}) {
        super.parseDependencies(json);

        for (const k1 in json) {
            if (k1 === "fields") {
                const fields = json[k1];
                for (const k2 in fields) {
                    const field = fields[k2];
                    switch (field["type"]) {
                        case "input":
                        case "select":
                        case "check":
                            for (const k of this.kids)
                                if (k.id == field["id"]) {
                                    k.parseDependencies(field);
                                    break;
                                }
                            break;
                    }
                }
            }
        }
    }
}
