/**
 * représentation sous forme d'arbre du formulaire et de ses éléments
 */
export abstract class FormulaireNode {
    /**
     * identifiant dans le fichier de conf
     */
    protected _confId: string;

    /**
    * id numérique unique
    */
    private _uid: number;

    private static _uidSequence: number = 0;

    /**
     * enfants (utilisé entre autres pour FormulaireDefinition, FieldSet et FieldsetContainer)
     */
    private _kids: FormulaireNode[];

    /**
     * true si fait partie d'un template
     */
    private _isTemplate: boolean;

    /**
    * élément d'origine si this est issu d'un template
    */
    private _fromTemplate: FormulaireNode;

    constructor(isTmpl: boolean = false) {
        this._kids = [];
        this._uid = FormulaireNode.uidGenerator();
        this._isTemplate = isTmpl;
    }

    get id(): string {
        return this._confId;
    }

    public get kids(): FormulaireNode[] {
        return this._kids;
    }

    public get isTemplate(): boolean {
        return this._isTemplate;
    }

    public get fromTemplate(): FormulaireNode {
        return this._fromTemplate;
    }

    private static uidGenerator(): number {
        let res = this._uidSequence;
        this._uidSequence++;
        return res;
    }

    public get uid() {
        return this._uid;
    }

    protected copyTo(n: FormulaireNode) {
        if (this._isTemplate) {
            n._isTemplate = false;
            n._fromTemplate = this;
        }

        n._confId = this._confId;
        for (const k of this._kids) {
            const nk = k.clone();
            k.copyMembers(nk);
            n._kids.push(nk);
        }
    }

    /**
     * obtient une copie en profondeur
     */
    public getDeepClone(): FormulaireNode {
        const res = this.clone();
        this.copyMembers(res);
        return res;
    }

    /**
     * cherche un FormulaireNode par son id de conf
     */
    public getFormulaireNodeById(id: string): FormulaireNode {
        if (this.id === id)
            return this;

        for (const k of this._kids) {
            const res = k.getFormulaireNodeById(id);
            if (res !== undefined)
                return res;
        }

        return undefined;
    }

    /**
     * cherche un FormulaireNode par son id numérique unique
     */
    public getFormulaireNodeByUid(uid: number): FormulaireNode {
        if (this.uid == uid)
            return this;

        for (const k of this._kids) {
            const res = k.getFormulaireNodeByUid(uid);
            if (res !== undefined)
                return res;
        }

        return undefined;
    }

    /**
     * crée une nouvelle instance (appel simple du constructeur)
     */
    protected abstract clone(): FormulaireNode;

    /**
     * copie des membres
     */
    protected abstract copyMembers(n: FormulaireNode);

    public abstract parseConfig(json: {}, data?: {});

    public abstract parseDependencies(json: {});
}
