import { cLog, Result, ResultElement } from "jalhyd";

import { CalculatorResults } from "./calculator-results";

export class RemousResults extends CalculatorResults {
    /**
     * hauteur de berge
     */
    private _hautBerge: ResultElement;

    /**
     * pente du fond
     */
    private _penteFond: number;

    /**
     * hauteur normale
     */
    private _hautNormale: ResultElement;

    /**
     * hauteur critique
     */
    private _hautCritique: ResultElement;

    /**
    * résultats numériques
    */
    private _series: ResultElement[] = [];

    /**
     * présence de données pour la courbe fluviale
     */
    private _hasFlu: boolean;

    /**
     * présence de données pour la courbe torrentielle
     */
    private _hasTor: boolean;

    /**
     * présence de données pour le paramètre supplémentaire
     */
    private _hasExtra: boolean;

    /**
     * le paramètre supplémentaire est affiché dans un graphe séparé
     */
    private _extraGraph: boolean;

    /**
     * titre de la colonne du paramètre supplémentaire
     */
    private _extraParamLabel: string;

    /**
     * journal de calcul
     */
    private _log: cLog;

    constructor() {
        super();
        this.reset();
    }

    public reset() {
        this._series = [];
        this._log = new cLog();
        this._hautBerge = undefined;
        this._penteFond = undefined;
        this._hautNormale = undefined;
        this._hautCritique = undefined;
        this._extraParamLabel = undefined;
        this._hasFlu = false;
        this._hasTor = false;
        this._hasExtra = false;
        this._extraGraph = false;
    }

    public get log() {
        return this._log;
    }

    public set log(l: cLog) {
        this._log = l;
    }

    public get series() {
        return this._series;
    }

    public addResult(res: Result) {
        if (res.ok) {
            const trX = res.getExtraResult("trX");
            const flu = res.getExtraResult("flu");
            const tor = res.getExtraResult("tor");
            const extra = res.getExtraResult("tRes");
            for (let x of trX) {
                let r = new ResultElement(x);
                if (flu[x] != undefined) {
                    r.addExtraResult("flu", flu[x]);
                    this._hasFlu = true;
                }
                if (tor[x] != undefined) {
                    r.addExtraResult("tor", tor[x]);
                    this._hasTor = true;
                }
                if (extra[x] != undefined) {
                    r.addExtraResult("extra", extra[x]);
                    this._hasExtra = true;
                }
                this._series.push(r);
            }
        }

        this._log.addLog(res.globalLog);
    }

    public get extraParamLabel() {
        return this._extraParamLabel;
    }

    public set extraParamLabel(l: string) {
        this._extraParamLabel = l;
    }

    public get hautBerge() {
        return this._hautBerge;
    }

    public set hauteurBerge(v: ResultElement) {
        this._hautBerge = v;
    }

    public get penteFond() {
        return this._penteFond;
    }

    public set penteFond(v: number) {
        this._penteFond = v;
    }

    public get hautNormale() {
        return this._hautNormale;
    }

    public set hauteurNormale(v: ResultElement) {
        this._hautNormale = v;
    }

    public get hautCritique() {
        return this._hautCritique;
    }

    public set hauteurCritique(v: ResultElement) {
        this._hautCritique = v;
    }

    public get hasExtra() {
        return this._hasExtra;
    }

    public get extraGraph() {
        return this._extraGraph;
    }

    public set extraGraph(b: boolean) {
        this._extraGraph = b;
    }

    public get hasResults(): boolean {
        return this._hautBerge != undefined ||
            this._penteFond != undefined ||
            this._hautNormale != undefined ||
            this._hautCritique != undefined ||
            this.hasSeriesData;
    }

    /**
     * @return true si il existe au moins une données dans la série
     */
    public get hasSeriesData(): boolean {
        return this._series.length > 0;
    }

    /**
     * @return true si il existe au moins une données dans la série fluviale
     */
    public get hasFluData(): boolean {
        return this._hasFlu;
    }

    /**
     * @return true si il existe au moins une données dans la série torrentielle
     */
    public get hasTorData(): boolean {
        return this._hasTor;
    }
}
