export interface Observer {
    update(sender: IObservable, data: any): void;
}

export interface IObservable {
    /**
     * ajoute un observateur à la liste
     */
    addObserver(o: Observer);

    /**
     * supprime un observateur de la liste
     */
    removeObserver(o: Observer);

    /**
     * notifie un événement aux observateurs
     */
    notifyObservers(data: any, sender?: any);
}

export class Observable implements IObservable {
    private _observers: Observer[];

    constructor() {
        this._observers = [];
    }

    /**
     * ajoute un observateur à la liste
     */
    public addObserver(o: Observer) {
        if (!this._observers.includes(o))
            this._observers.push(o);
    }

    /**
     * supprime un observateur de la liste
     */
    public removeObserver(o: Observer) {
        this._observers = this._observers.filter(a => a !== o);
    }

    /**
     * notifie un événement aux observateurs
     */
    public notifyObservers(data: any, sender?: any) {
        if (sender == undefined)
            sender = this;
        for (let o of this._observers)
            o.update(sender, data);
    }
}
